-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

--Central Tire Inflation system by TrackpadUser with help from Diamondback
--This thing is only compatible with 4 wheeled vehicles, but is probably easily modifiable for other applications

local M = {}
M.type = "auxilliary"
M.relevantDevice = nil

--Wheel object
local frontWheels = {}
local rearWheels = {}

--Control variables
local frontBasePressure = 0
local frontSideStiffFactor = 0
local frontReinforceStiffFactor = 0
local rearBasePressure = 0
local dtcounter = 0

local previousInput = 1

--Minimum pressure constant in PSI. 
--Should be at least above 1 to keep the tire from imploding due to being a negative relative pressure.
--Adjust based on testing, to keep typical tires from collapsing or reversing on impacts with kerbs and jumps.
local minPressurePSI = 3

local function updateGFX(dt) -- ms

	--adjusts tire pressure based on the pressureCoef variation, which is managed in the tireInflationElectrics.lua file
	for _,wheel in pairs(frontWheels) do
        obj:setGroupPressure(v.data.pressureGroups[wheel.pressureGroup], (frontBasePressure-minPressurePSI)*electrics.values['pressureCoef']*6894.757+101325+6894.757*minPressurePSI)
		if wheel.sideBeams ~= nil then
			for _, beamcid in pairs(wheel.sideBeams) do
				obj:setBeamSpringDamp(beamcid, v.data.beams[beamcid].beamSpring * electrics.values['pressureCoef']-0.05+v.data.beams[beamcid].beamSpring*0.05, v.data.beams[beamcid].beamDamp, -1, -1)
			end
		end
	end
	for _,wheel in pairs(rearWheels) do
        obj:setGroupPressure(v.data.pressureGroups[wheel.pressureGroup], (rearBasePressure-minPressurePSI)*electrics.values['pressureCoef']*6894.757+101325+6894.757*minPressurePSI)
		if wheel.sideBeams ~= nil then
			for _, beamcid in pairs(wheel.sideBeams) do
				obj:setBeamSpringDamp(beamcid, v.data.beams[beamcid].beamSpring * electrics.values['pressureCoef']-0.05+v.data.beams[beamcid].beamSpring*0.05, v.data.beams[beamcid].beamDamp, -1, -1)
			end
		end
	end
	
	if previousInput ~= electrics.values['pressure_input'] and electrics.values['pressure_input'] == 0 then
		--print(math.floor(((obj:getGroupPressure(v.data.pressureGroups[frontWheels[1].pressureGroup])-101325)/6894.757)*100+0.5)/100)
		gui.message("Tire pressure set to " .. (math.floor(((obj:getGroupPressure(v.data.pressureGroups[frontWheels[1].pressureGroup])-101325)/6894.757)*100+0.5)/100) .. " PSI")
	end
	
	previousInput = electrics.values['pressure_input']
			
	--debug print every 1 second
	if dtcounter == 60 then
	--	print(frontWheels[1].wheelSideBeamSpring)
	--	print(obj:getGroupPressure(v.data.pressureGroups[frontWheels[1].pressureGroup]))
	--	print(electrics.values['pressureCoef'])
		dtcounter =0
	else
		dtcounter = dtcounter+1
	end 
end

local function init(jbeamData)

	--Initialises the wheels and puts them in the correct tables based on their name
	for _,wd in pairs(v.data.wheels) do
		if string.match(wd.name,"FR") then table.insert(frontWheels, wd) end
		if string.match(wd.name,"FL") then table.insert(frontWheels, wd) end
		if string.match(wd.name,"RL") then table.insert(rearWheels, wd) end
		if string.match(wd.name,"RR") then table.insert(rearWheels, wd) end
	end
	
	--Gets the initial pressure of the tire for future reference
	frontBasePressure = frontWheels[1].pressurePSI
	rearBasePressure = rearWheels[1].pressurePSI
end

M.init = init
M.reset = nop
M.updateGFX = updateGFX

return M